<?php
/*
Plugin Name: Light Formatter
Plugin URI: http://hello.lumiere-couleur.com/light_formatter/
Description: Very simple contents formatter.
Author: Author: smilkobuta
Version: 0.2
Author URI: http://hello.lumiere-couleur.com/
*/

class LightFormatter {
    var $pattern;

    function LightFormatter() {
        $this->setup();
        if (function_exists('add_filter')) {
            add_filter('the_content', array(&$this, 'format'), 9);
        }
    }

    function setup() {
        $this->add_pattern('contents', '/#contents(.*)/ms', array(&$this, 'format_contents'));
        $this->add_pattern('indent', '/^(\^+)/m', array(&$this, 'format_indent'));
        $this->add_pattern('br', '/(~+)(((<\/[^<>]+>)+)?([\r\n]*))$/m', array(&$this, 'format_br'));
        $this->add_pattern('table', '/^(?:(?:<[^<>]+>)+)?\s*(\|(?:(?!\n\s*[^|]).)*\|)\s*$/ms', array(&$this, 'format_table'));
    }

    function add_pattern($name, $regex, $callback) {
        $this->pattern[$name] = array($regex, $callback);
    }

    function format($str) {
        if (is_array($this->pattern)) {
            foreach ($this->pattern AS $name => $pattren) {
                $str = preg_replace_callback($pattren[0]
                    , $pattren[1]
                    , $str);
            }
        }
        return $str;
    }

    function format_contents($match) {
        $contents = $match[1];
        $list = '';

        // 見出し（h1～h6）が使われている場合、目次を作成
        if (preg_match_all('/<h([1-6])>((?:(?!\/h[1-6]).)*)<\/h[1-6]>/', $contents, $matches)) {
            // h1～h6のうち、使われているナンバーを取得
            $all_headernum = array_unique($matches[1]);
            sort($all_headernum);
            $all_headernum = array_flip($all_headernum);

            // リストの出力
            $list .= "<div id=\"lfcontents\">";
            $parent_pos = array();
            $current_level = -1;
            $current_pos = -1;
            $countheader = 0;
            foreach ($matches[2] AS $k => $v) {
                $headernum = $matches[1][$k];
                if ($current_level < $all_headernum[$headernum]) {
                    // 階層を下る
                    $repeat = $all_headernum[$headernum] - $current_level;
                    for ($i = 0; $i < $repeat; $i++) {
                        $parent_pos[$current_level] = $current_pos;
                        $current_level ++;
                        $current_pos = 0;

                        // リストスタート
                        $list .= "\n<ul class=\"lfcontents_level$current_level\">\n";
                    }
                } else if ($current_level > $all_headernum[$headernum]) {
                    // 階層を上がる
                    $repeat = $current_level - $all_headernum[$headernum];
                    for ($i = 0; $i < $repeat; $i++) {
                        unset($parent_pos[$current_level]);
                        $current_level --;
                        $current_pos = $parent_pos[$current_level];

                        // リストエンド
                        $list .= "</ul>\n";
                        $list .= "</li>\n";
                    }
                } else {
                    $current_pos ++;
                }

                // リスト出力
                $aname = "lf_list_" . intval($countheader++);
                $list .= "<li><a href=\"#$aname\">" . strip_tags($v) . "</a>";

                // 次の項目が同じレベルもしくは上位レベルの場合、すぐにリストを閉じる
                if (!$matches[2][$k + 1] || $matches[1][$k + 1] <= $headernum) {
                    $list .= "</li>\n";
                }
            }

            // リストの出力後片付け
            for ($i = 0; $i < $current_level + 1; $i++) {
                // リストエンド
                $list .= "</ul>\n";
            }
            $list .= "</div>";

            // h1～h6の後ろにアンカーをつける。
            $contents = preg_replace_callback('/(<h[1-6]>)((?:(?!\/h[1-6]).)*)(<\/h[1-6]>)/',
                create_function(
                    '$matches',
                    '
                        global $__lf_countheader;
                        $aname = "lf_list_" . intval($__lf_countheader++);
                        $ret = $matches[1] . $matches[2] . " <a href=\"#lfcontents\" style=\"font-size:12px;\" name=\"$aname\">&uarr;</a>" . $matches[3];
                        return $ret;
                    '
                ), $contents);
        }

        return $list . $contents;
    }

    function format_indent($match) {
        return str_repeat('&nbsp;&nbsp;', strlen($match[1]));
    }

    function format_br($match) {
        $ret = '';
        if (strlen($match[1]) > 1) {
            $ret = str_repeat('<br >', strlen($match[1]) - 1);
        } else {
            $ret = "&nbsp;";
        }
        $ret .= $match[2];
        return $ret;
    }

    function format_table($match) {
        $str = trim($match[1]);
        $ret = '';
        $rows = split("\n", $str);
        $global_options = array();
        $flg_tabletag = false;

        if (count($rows) > 0) {
            $rows = array_map('trim', $rows);

            // perform ROWS
            foreach ($rows AS $i => $row) {
                $row = preg_replace('/(^\s*\||\|\s*$)/', '', $row);

                // get global options
                if ($i == 0 && substr($row, 0, 2) == '##') {
                    $global_options = $this->_parse_options(substr($row, 2));
                    continue;
                }

                $options = $global_options;

                // print table tag
                if (!$flg_tabletag) {
                    $options['class'] = $options['class'] ? $options['class'] . ' lftable' : 'lftable';
                    $ret .= $this->_add_attr('table', array('id' => $options['id'], 'style' => $options['table_style'], 'class' => $options['class']));
                    $flg_tabletag = true;
                }

                // print tr tag
                $ret .= $this->_add_attr('tr', array('style' => $options['tr_style']
                                            , 'class' => $options['tr_class']));

                $cols = split('\|', $row);
                $cols = array_map('trim', $cols);

                // idとclassをクリア
                unset($options['id']);
                unset($options['class']);

                // perform COLS
                foreach ($cols AS $j => $col) {
                    $flg_header = false;
                    if (substr($col, 0, 1) == '~') {
                        $flg_header = true;
                        $col = substr($col, 1);
                    }

                    $local_options = $options;

                    // get local options
                    if (preg_match('/^##(.*)##(.*)/', $col, $match)) {
                        $col = $match[2];
                        $local_options = array_merge($local_options, $this->_parse_options($match[1]));
                    }

                    if ($flg_header) {
                        // print th tag
                        $ret .= $this->_add_attr('th', array('id' => $local_options['id']
                                                    , 'style' => join(';', array_filter(array($local_options['style'], $local_options['th_style'])))
                                                    , 'class' => join(';', array_filter(array($local_options['class'], $local_options['th_class'])))
                                                ));
                        $ret .= "$col</th>\n";
                    } else {
                        // print td tag
                        $ret .= $this->_add_attr('td', array('id' => $local_options['id']
                                                    , 'style' => join(';', array_filter(array($local_options['style'], $local_options['td_style'])))
                                                    , 'class' => join(';', array_filter(array($local_options['class'], $local_options['td_class'])))
                                                ));
                        $ret .= "$col</td>\n";
                    }
                }
                $ret .= "</tr>\n";
            }
            $ret .= "</table>";
        }
        return $ret;
    }

    function _add_attr($tag, $attrs, $default_attr = '') {
        $ret = "<$tag" . ($default_attr ? " $default_attr" : '');
        foreach ($attrs AS $k => $v) {
            if ($v) {
                $ret .= " $k=\"" . $this->_remove_xss($v) . "\"";
            }
        }
        $ret .= ">";
        return $ret;
    }

    function _remove_xss($str) {
        $str = htmlspecialchars($str);                  // HTMLエンティティ
        $str = str_replace('\\', '', $str);             // ASCII指定削除
        $str = preg_replace('/\/\*|\*\//', '', $str);   // コメント削除
        $str = mb_convert_kana($str, 'as', 'UTF-8');    // 全角→半角
        $str = preg_replace('/expression/i', '', $str); // 特定CSS属性削除
        $str = preg_replace('/exec/i', '', $str);       // 特定CSS属性削除
        $str = preg_replace('/eval/i', '', $str);       // 特定CSS属性削除
        return $str;
    }

    function _parse_options($str) {
        $ret = array();
        $_options = split(',',$str);
        foreach ($_options AS $option) {
            list($k, $v) = split('=', $option);
            $ret[trim($k)] = trim($v);
        }
        return $ret;
    }
}

$light_formatter = new LightFormatter();
?>